@@ -1,4 +1,4 @@
-# Copyright (C) 2019-2021, TomTom (http://tomtom.com).
+# Copyright (C) 2019, TomTom (http://tomtom.com).
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -15,8 +15,14 @@
 Tests for the `asciidoxy.model` module.
 """
 
-from asciidoxy.model import (Compound, Parameter, ReferableElement, ReturnValue, ThrowsClause,
-                             TypeRef)
+from asciidoxy.model import (
+    Compound,
+    Parameter,
+    ReferableElement,
+    ReturnValue,
+    ThrowsClause,
+    TypeRef,
+)
 
 
 def test_minimal_constructed_repr():
@@ -488,8 +494,7 @@ def test_compound__init__default():
 
     assert compound.brief == ""
     assert compound.description == ""
-    assert compound.precondition == ""
-    assert compound.postcondition == ""
+    assert compound.sections == {}
 
     assert compound.static is False
     assert compound.const is False
@@ -521,8 +526,7 @@ def test_compound__init__positional():
 
     assert compound.brief == ""
     assert compound.description == ""
-    assert compound.precondition == ""
-    assert compound.postcondition == ""
+    assert compound.sections == {}
 
     assert compound.static is False
     assert compound.const is False
@@ -549,13 +553,12 @@ def test_compound__init__keyword():
                         initializer=" = 2",
                         brief="brief",
                         description="description",
+                        sections={"section_name": "section_text"},
                         static=True,
                         const=True,
                         deleted=True,
                         default=True,
-                        constexpr=True,
-                        precondition="pre",
-                        postcondition="post")
+                        constexpr=True)
 
     assert compound.id == "id"
     assert compound.name == "name"
@@ -582,8 +585,7 @@ def test_compound__init__keyword():
 
     assert compound.brief == "brief"
     assert compound.description == "description"
-    assert compound.precondition == "pre"
-    assert compound.postcondition == "post"
+    assert compound.sections == {"section_name": "section_text"}
 
     assert compound.static is True
     assert compound.const is True
@@ -635,13 +637,12 @@ def test_compound__eq__full():
                      initializer=" = 2",
                      brief="brief",
                      description="description",
+                     sections={"section_name": "section_text"},
                      static=True,
                      const=True,
                      deleted=True,
                      default=True,
-                     constexpr=True,
-                     precondition="pre",
-                     postcondition="post")
+                     constexpr=True)
     second = Compound(id="id",
                       name="name",
                       full_name="full_name",
@@ -659,20 +660,18 @@ def test_compound__eq__full():
                       initializer=" = 2",
                       brief="brief",
                       description="description",
+                      sections={"section_name": "section_text"},
                       static=True,
                       const=True,
                       deleted=True,
                       default=True,
-                      constexpr=True,
-                      precondition="pre",
-                      postcondition="post")
+                      constexpr=True)
 
     assert first == second
     assert second == first
 
     for attr_name in ("id", "name", "full_name", "language", "kind", "include", "namespace", "prot",
-                      "definition", "args", "initializer", "brief", "description", "precondition",
-                      "postcondition"):
+                      "definition", "args", "initializer", "brief", "description"):
         setattr(second, attr_name, "other")
         assert first != second
         assert second != first
@@ -703,3 +702,13 @@ def test_compound__eq__full():
     assert first != second
     assert second != first
     second.returns.description = first.returns.description
+
+    second.sections["section_name"] = "other"
+    assert first != second
+    assert second != first
+    second.sections = first.sections.copy()
+
+    second.sections["other_section_name"] = "other"
+    assert first != second
+    assert second != first
+    second.sections = first.sections.copy()
